@@ -72,7 +72,7 @@ class BackupManager(object):
             "servicenowCMDBPolicies", "vraPolicies"
         ]
 
-        if version.parse(self.ofm.onefuse_version) > version.parse('1.4'):
+        if version.parse(self.ofm.onefuse_version) >= version.parse('1.4'):
             self.policy_types.insert(0, 'modules')
             self.policy_types.insert(1, 'connectionInfo')
             self.policy_types.append('modulePolicies')
@@ -137,7 +137,10 @@ class BackupManager(object):
                                   f'{policy["name"]}')
             file_path = f'{backups_path}{policy_type}{path_char}'
             if policy_type == 'modules':
-                self.ofm.export_pluggable_module(policy["name"], file_path)
+                # Modules will overwrite existing modules in the file path if
+                # found
+                self.ofm.export_pluggable_module(policy["name"], file_path,
+                                                 True)
                 continue
             if policy_type == "endpoints":
                 if "credential" in policy["_links"]:
@@ -276,7 +279,6 @@ class BackupManager(object):
             Dict of JSON of a policy that you are looking to restore
         """
         restore_json = {}
-        version = self.ofm.get_onefuse_version()
         for key in json_content:
             if key == "_links":
                 for key2 in json_content["_links"]:
